// AUTOmatix 3D
// by Sebastian Reichelt
// Main Source File

#include <doors.h>
#include <all.h>
#include "Buffered Graphics.h"

int _ti89;        // Produce .89Z file

char _comment [] = "AUTOmatix 3D by Sebastian Reichelt";

TGraphBuffer *GraphBuffer;   // Graphics buffer to draw in

// This program substitutes OneSubst for 1 in certain floating point operations.
// Higher values provide more accurate display
#define OneSubst 25

// Distance from which you can see
#define VisiblePos 10

// Game speed factor (higher value = slower gameplay)
#define TimeFactor 25

int RoadHeight, RoadWidth, HalfWidth;

unsigned int Time;
int CarPos, CarXPos, CarTurn, CarHeight, CarSlope, CarSpeed;
unsigned int LastSpeedChangeTime, LastTurnTime;
int LastSpeedChangeCarPos, LastTurnCarXPos;

// Functions
void RunLines();

unsigned int GetDist(unsigned int X);
int GetRoadX(unsigned long Dist);
int GetRoadY(unsigned long Dist);

SCR_RECT *ScreenRect;
SCR_RECT ScrnRect;

// Main Function
int _main()
{
	int LastTurn;
	
	ClrScr();
	
	// Get actual display size (with status bar)
	ScrnRect = *ScrRect;
	if (ScrnRect.xy.x1 == 239)
		ScrnRect.xy.y1 = 127;
	else
		ScrnRect.xy.y1 = 99;
	ScreenRect = &ScrnRect;
	
	// Initialize for grayscale graphics
	if (GrayMode (GRAY_ON))
		SetCurGrayPlanes (2);
	else
		SetCurGrayPlanes (1);
	GraphBuffer = malloc (sizeof (TGray4Buffer));
	InitBuf (GraphBuffer, GetCurGrayPlanes());
	FlushBuf (GraphBuffer);
	
	RoadHeight = ScreenRect->xy.y1 / 2;
	RoadWidth = ScreenRect->xy.x1 + 40;
	HalfWidth = ScreenRect->xy.x1 / 2;
	
	CarTurn = 0;
	CarXPos = 0;
	CarPos = 0;
	CarSpeed = 220;
	LastSpeedChangeTime = 0;
	LastSpeedChangeCarPos = 0;
	LastTurnTime = 0;
	LastTurnCarXPos = 0;
	for (Time = 0; Time < 50; Time++)
	{
		LastTurn = CarTurn;
		if (LastTurnTime < LastSpeedChangeTime)
		{
			LastTurnTime = Time;
			LastTurnCarXPos = CarXPos;
		}
		CarPos = LastSpeedChangeCarPos + (Time - LastSpeedChangeTime) * CarSpeed / TimeFactor;
		CarXPos = LastTurnCarXPos + (Time - LastTurnTime) * CarSpeed / TimeFactor * CarTurn / OneSubst;
		RunLines();
		if (CarTurn != LastTurn)
		{
			LastTurnTime = Time;
			LastTurnCarXPos = CarXPos;
		}
	}
	
	ngetchx();
	
	free (GraphBuffer);
	GrayMode (GRAY_OFF);
	ClrScr();
	return 0;
}

// Other Functions
void RunLines()
{
	int XPos, YPos, ScrX, ScrY, LastScrX = 0, LastScrY = -1, CurScrX, TrapLoop, Mul, Horizon, ScrW, LastScrW = 0, CurScrW, I, Dist;
	unsigned long Distance;
	TColor RoadAttr = 0, SideAttr = 0;
	
	CarHeight = 0;
	CarHeight = -GetRoadY (CarPos);
	CarSlope = GetRoadY (CarPos + OneSubst) - GetRoadY (CarPos);
	CarTurn = GetRoadX (CarPos + OneSubst) - GetRoadX (CarPos);
	Horizon = ScreenRect->xy.y1 - RoadHeight + CarSlope;
	
	DrawFullHLinesBuf (GraphBuffer, 0, Horizon, clWhite);
	
	for (I = OneSubst / VisiblePos; I <= OneSubst; I++)
	{
		Dist = GetDist (I);
		Distance = Dist;
		Distance += CarPos;
		if ((((Dist + CarPos) * 2 / OneSubst) % 2) == 0)
		{
			RoadAttr = clGray;
			SideAttr = clSilver;
		}
		else
		{
			RoadAttr = clBlack;
			SideAttr = clGray;
		}
		XPos = (GetRoadX (Distance) - CarXPos - Dist * CarTurn / OneSubst) * I / OneSubst;
		YPos = (GetRoadY (Distance) + CarHeight - RoadHeight - Dist * CarSlope / OneSubst) * I / OneSubst;
		ScrX = HalfWidth + XPos;
		ScrY = ScreenRect->xy.y1 - RoadHeight - YPos;
		ScrW = RoadWidth / 2 * I / OneSubst;
		
		if (ScrY != LastScrY)
		{
			if (LastScrY == -1)
				LastScrY = Horizon;
			
			if (ScrY < LastScrY)
				Mul = -1;
			else
				Mul = 1;
			
			if ((LastScrW == 0) || ((XPos > HalfWidth + RoadWidth / 2) || (XPos < -HalfWidth - RoadWidth / 2)))
				DrawFullHLinesBuf (GraphBuffer, LastScrY, ScrY, clGray);
			else
			{
				for (TrapLoop = LastScrY + Mul; Mul * TrapLoop <= Mul * ScrY; TrapLoop += Mul)
				{
					if ((TrapLoop >= ScreenRect->xy.y0) && (TrapLoop <= ScreenRect->xy.y1))
					{
						//       {last pos}  {distance of Xs }   {       returns a loop value between 0 and 1        }
						CurScrX = LastScrX + (ScrX - LastScrX) * (TrapLoop - LastScrY - Mul) / (ScrY - LastScrY) * Mul;
						CurScrW = LastScrW + (ScrW - LastScrW) * (TrapLoop - LastScrY - Mul) / (ScrY - LastScrY) * Mul;
						DrawHLineBuf (GraphBuffer, ScreenRect->xy.x0, CurScrX - CurScrW - 1, TrapLoop, SideAttr);
						DrawHLineBuf (GraphBuffer, CurScrX - CurScrW, CurScrX + CurScrW, TrapLoop, RoadAttr);
						DrawHLineBuf (GraphBuffer, CurScrX + CurScrW + 1, ScreenRect->xy.x1, TrapLoop, SideAttr);
					}
				}
			}
			
			LastScrX = ScrX;
			LastScrY = ScrY;
			LastScrW = ScrW;
		}
	}
	
	FlushBuf (GraphBuffer);
}

unsigned int GetDist(unsigned int X)
{
	return (OneSubst * OneSubst / X - OneSubst);
}

int GetRoadX(unsigned long Dist)
{
	int PartDist;
	if (Dist < 10000)
	{
		PartDist = Dist;
		if (Dist < OneSubst * 8)
			return 0;
		else
			return (PartDist - OneSubst * 8);
	}
	else
		return 0;
}

int GetRoadY(unsigned long Dist)
{
	int PartDist;
	// General Formula: a(x+b)^2+c = a * (PartDist + b * OneSubst) * (PartDist + b * OneSubst) / OneSubst + c * OneSubst
	//                  where x = Dist / OneSubst  (the actual distance regardless of what OneSubst is)
	if (Dist < OneSubst * 2)
	{
		PartDist = Dist;
		return (PartDist * PartDist / 4 / OneSubst);  // Formula x^2/4
	}
	else if (Dist < OneSubst * 6)
	{
		PartDist = Dist;
		PartDist -= OneSubst * 4;
		return (-PartDist * PartDist / 4 / OneSubst + 2 * OneSubst);  // Formula -(x-4)^2/4+2
	}
	else if (Dist < OneSubst * 8)
	{
		PartDist = Dist;
		PartDist -= OneSubst * 8;
		return (PartDist * PartDist / 4 / OneSubst);  // Formula (x-8)^2/4
	}
	else
		return 0;
}
